package com.example.demo.user.service.impl;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.example.demo.common.utils.Md5Util;
import com.example.demo.common.utils.RedisClient;
import com.example.demo.common.utils.TokenUtils;
import com.example.demo.user.domain.User;
import com.example.demo.user.dto.LoginDto;
import com.example.demo.user.dto.PhoneDto;
import com.example.demo.user.dto.UserDto;
import com.example.demo.user.service.UserService;
import com.example.demo.user.dao.UserMapper;
import com.example.demo.user.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
* @author 徐乐
* @description 针对表【user(用户基础信息表)】的数据库操作Service实现
* @createDate 2025-06-04 17:31:35
*/
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User>
    implements UserService{

    @Autowired
    private UserMapper userMapper;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    @Autowired
    private RedisClient redisClient;
    @Override
    public ResultVo register(UserDto dto) {
        QueryWrapper<User> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("phone", dto.getPhone());
        log.info("加锁");
        RLock lock = redissonClient.getLock(dto.getPhone());
        try {
            log.info("获取锁");
            boolean lockSuccess = lock.tryLock(3, TimeUnit.SECONDS);
            if (!lockSuccess) {
                throw new RuntimeException("操作频繁,请稍后再试");
            }
            User user = userMapper.selectOne(userQueryWrapper);
            if (user != null) {
                return ResultVo.error("用户已存在");
            }
            log.info("注册");
            User u = new User();
            u.setId(IdUtil.getSnowflake().nextId());
            //生成盐值
            String uuid = UUID.randomUUID().toString().replaceAll("-", "");
            u.setSalt(uuid);
            //获取密码
            String password = dto.getPassword();
            //加密
            String md5Password = Md5Util.MD5(uuid + password);
            u.setPassword(md5Password);
            u.setPhone(dto.getPhone());
            userMapper.insert(u);
            return ResultVo.success("注册成功");
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("注册过程中出现异常,请稍后再试");
        } finally {
            if (lock.isLocked() && lock.isHeldByCurrentThread()) {
                log.info("释放锁");
                lock.unlock();
            }
        }
    }

    /**
     * 统一获取验证码方法
     * @param phone 手机号
     * @return 验证码
     */
    private Object getVerificationCode(String phone) {
        String key = "CODE:" + phone;
        return redisClient.get(key);
    }

    /**
     * 统一删除验证码方法
     * @param phone 手机号
     */
    private void deleteVerificationCode(String phone) {
        String key = "CODE:" + phone;
        redisClient.del(key);
    }

    @Override
    public ResultVo login(LoginDto dto) {
        QueryWrapper<User> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("phone",dto.getPhone());
        User user = userMapper.selectOne(userQueryWrapper);
        if (user==null){
            return ResultVo.error("用户不存在");
        }
        //获取盐值
        String salt = user.getSalt();
        //获取密码
        String password = dto.getPassword();
        //加密
        String encryptedPassword = Md5Util.MD5(salt + password);
        if (encryptedPassword.equals(user.getPassword())){
            String token = TokenUtils.createJwtToken(user.getId().toString());
            redisTemplate.opsForValue().set("userToken:",token);
            return ResultVo.success(token);
        }
        return ResultVo.error("用户名或密码有误");
    }

    @Override
    public ResultVo phoneLogin(PhoneDto dto) {
        // 参数校验
        if (dto.getPhone() == null ||!dto.getPhone().matches("1[3-9]\\d{9}")) {
            return ResultVo.error(400, "手机号格式错误");
        }

        // 获取验证码
        Integer collect = (Integer) getVerificationCode(dto.getPhone());
        if (collect == null) {
//            logger.info("手机号 {} 验证码已过期", dto.getPhone());
            return ResultVo.error(201, "验证码已过期");
        }

        // 验证验证码
        String codeStr = String.valueOf(collect);
        String userCode = dto.getCode();
        if (userCode == null ||!userCode.matches("\\d{6}")) {
//            logger.info("手机号 {} 输入的验证码格式错误", dto.getPhone());
            return ResultVo.error(201, "验证码格式错误");
        }

        if (!codeStr.equals(userCode)) {
//            logger.info("手机号 {} 验证码错误，Redis: {}, 用户输入: {}",
//                    dto.getPhone(), codeStr, userCode);
            return ResultVo.error(201, "验证码错误");
        }

        // 验证通过后删除验证码
        deleteVerificationCode(dto.getPhone());


        // 先判断账号是否存在
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(User::getPhone, dto.getPhone());
        User user = userMapper.selectOne(queryWrapper);
//        if (user == null) {
//            user.setUsername("随机账户"+String.valueOf(IdUtil.getSnowflake().nextId()));
//            user.setId(IdUtil.getSnowflake().nextId());
//            userMapper.insert(user);
//            redisClient.set("uid",  user.getId());
//            redisClient.expire("uid", 60*60);
//            Long id = user.getId();
//            String token = AppJwtUtil.getToken(id);
//            redisClient.set("token",  token);
//            Map<String, String> map = new HashMap<>();
//            map.put("token",token);
//            return ResultVo.success(map);
//        }
//        redisClient.set("uid",  user.getId());
//        redisClient.expire("uid", 60*60);
        String token = TokenUtils.createJwtToken(user.getId().toString());
        redisTemplate.opsForValue().set("userToken:",token);
        return ResultVo.success(token);
    }
}




